@@ -6,7 +6,7 @@
 #include "Compadre_Functors.hpp"
 #include "Compadre_CreateConstraints.hpp"
 
-#include "KokkosBatched_Gemm_TeamVector_Internal.hpp"
+#include "KokkosBatched_Gemm_Decl.hpp"
 
 namespace Compadre {
 
@@ -265,12 +265,15 @@ void GMLS::generatePolynomialCoefficients(const int number_of_batches, const boo
                 if (_dense_solver_type == DenseSolverType::LU) {
                     // solves P^T*P against P^T*W with LU, stored in P
                     Kokkos::Profiling::pushRegion("Curvature LU Factorization");
-                    GMLS_LinearAlgebra::batchLUFactorize(_pm, _RHS.data(), RHS_dim_0, RHS_dim_1, _P.data(), P_dim_1, P_dim_0, manifold_NP, manifold_NP, _max_num_neighbors, this_batch_size);
+                    // batchLU expects layout_left matrix tiles for B
+                    // by giving it layout_right matrix tiles with reverse ordered ldb and ndb
+                    // it effects a transpose of _P in layout_left
+                    GMLS_LinearAlgebra::batchQRPivotingSolve<layout_right,layout_left,layout_right>(_pm, _RHS.data(), RHS_dim_0, RHS_dim_1, _P.data(), P_dim_1, P_dim_0, manifold_NP, manifold_NP, _max_num_neighbors, this_batch_size);
                     Kokkos::Profiling::popRegion();
                 } else {
                     // solves P*sqrt(weights) against sqrt(weights)*Identity with QR, stored in RHS
-                    Kokkos::Profiling::pushRegion("Curvature QR Factorization");
-                    GMLS_LinearAlgebra::batchQRFactorize(_pm, _P.data(), P_dim_0, P_dim_1, _RHS.data(), RHS_dim_0, RHS_dim_1, _max_num_neighbors, manifold_NP, _max_num_neighbors, this_batch_size);
+                    Kokkos::Profiling::pushRegion("Curvature QR+Pivoting Factorization");
+                    GMLS_LinearAlgebra::batchQRPivotingSolve<layout_right,layout_right,layout_right>(_pm, _P.data(), P_dim_0, P_dim_1, _RHS.data(), RHS_dim_0, RHS_dim_1, _max_num_neighbors, manifold_NP, _max_num_neighbors, this_batch_size);
                     Kokkos::Profiling::popRegion();
                 }
 
@@ -294,12 +297,12 @@ void GMLS::generatePolynomialCoefficients(const int number_of_batches, const boo
             if (_dense_solver_type == DenseSolverType::LU) {
                 // solves P^T*P against P^T*W with LU, stored in P
                 Kokkos::Profiling::pushRegion("Curvature LU Factorization");
-                GMLS_LinearAlgebra::batchLUFactorize(_pm, _RHS.data(), RHS_dim_0, RHS_dim_1, _P.data(), P_dim_1, P_dim_0, manifold_NP, manifold_NP, _max_num_neighbors, this_batch_size);
+                GMLS_LinearAlgebra::batchQRPivotingSolve<layout_right,layout_left,layout_right>(_pm, _RHS.data(), RHS_dim_0, RHS_dim_1, _P.data(), P_dim_1, P_dim_0, manifold_NP, manifold_NP, _max_num_neighbors, this_batch_size);
                 Kokkos::Profiling::popRegion();
             } else {
                  // solves P*sqrt(weights) against sqrt(weights)*Identity, stored in RHS
-                Kokkos::Profiling::pushRegion("Curvature QR Factorization");
-                GMLS_LinearAlgebra::batchQRFactorize(_pm, _P.data(), P_dim_0, P_dim_1, _RHS.data(), RHS_dim_0, RHS_dim_1, _max_num_neighbors, manifold_NP, _max_num_neighbors, this_batch_size);
+                Kokkos::Profiling::pushRegion("Curvature QR+Pivoting Factorization");
+                GMLS_LinearAlgebra::batchQRPivotingSolve<layout_right,layout_right,layout_right>(_pm, _P.data(), P_dim_0, P_dim_1, _RHS.data(), RHS_dim_0, RHS_dim_1, _max_num_neighbors, manifold_NP, _max_num_neighbors, this_batch_size);
                 Kokkos::Profiling::popRegion();
            }
 
@@ -320,18 +323,13 @@ void GMLS::generatePolynomialCoefficients(const int number_of_batches, const boo
             _pm.CallFunctorWithTeamThreads<AssembleManifoldPsqrtW>(*this, this_batch_size);
 
             // solves P*sqrt(weights) against sqrt(weights)*Identity, stored in RHS
-            // uses SVD if necessary or if explicitly asked to do so (much slower than QR)
-            if (_nontrivial_nullspace || _dense_solver_type == DenseSolverType::SVD) {
-                Kokkos::Profiling::pushRegion("Manifold SVD Factorization");
-                GMLS_LinearAlgebra::batchSVDFactorize(_pm, _P.data(), P_dim_0, P_dim_1, _RHS.data(), RHS_dim_0, RHS_dim_1, max_num_rows, this_num_cols, max_num_rows, this_batch_size);
-                Kokkos::Profiling::popRegion();
-            } else if (_dense_solver_type == DenseSolverType::LU) {
+            if (_dense_solver_type == DenseSolverType::LU) {
                 Kokkos::Profiling::pushRegion("Manifold LU Factorization");
-                GMLS_LinearAlgebra::batchLUFactorize(_pm, _RHS.data(), RHS_dim_0, RHS_dim_1, _P.data(), P_dim_1, P_dim_0, this_num_cols, this_num_cols, max_num_rows, this_batch_size);
+                GMLS_LinearAlgebra::batchQRPivotingSolve<layout_right,layout_left,layout_right>(_pm, _RHS.data(), RHS_dim_0, RHS_dim_1, _P.data(), P_dim_1, P_dim_0, this_num_cols, this_num_cols, max_num_rows, this_batch_size);
                 Kokkos::Profiling::popRegion();
             } else {
-                Kokkos::Profiling::pushRegion("Manifold QR Factorization");
-                GMLS_LinearAlgebra::batchQRFactorize(_pm, _P.data(), P_dim_0, P_dim_1, _RHS.data(), RHS_dim_0, RHS_dim_1, max_num_rows, this_num_cols, max_num_rows, this_batch_size);
+                Kokkos::Profiling::pushRegion("Manifold QR+Pivoting Factorization");
+                GMLS_LinearAlgebra::batchQRPivotingSolve<layout_right,layout_right,layout_right>(_pm, _P.data(), P_dim_0, P_dim_1, _RHS.data(), RHS_dim_0, RHS_dim_1, max_num_rows, this_num_cols, max_num_rows, this_batch_size);
                 Kokkos::Profiling::popRegion();
             }
             Kokkos::fence();
@@ -348,32 +346,18 @@ void GMLS::generatePolynomialCoefficients(const int number_of_batches, const boo
             Kokkos::fence();
 
             // solves P*sqrt(weights) against sqrt(weights)*Identity, stored in RHS
-            // uses SVD if necessary or if explicitly asked to do so (much slower than QR)
-            if (_constraint_type != ConstraintType::NO_CONSTRAINT) {
-                if (_nontrivial_nullspace || _dense_solver_type == DenseSolverType::SVD) {
-                     Kokkos::Profiling::pushRegion("SVD Factorization");
-                     GMLS_LinearAlgebra::batchSVDFactorize(_pm, _RHS.data(), RHS_dim_0, RHS_dim_1, _P.data(), P_dim_1, P_dim_0, this_num_cols + added_coeff_size, this_num_cols + added_coeff_size, max_num_rows + _added_alpha_size, this_batch_size);
-                     Kokkos::Profiling::popRegion();
-                } else {
-                     Kokkos::Profiling::pushRegion("LU Factorization");
-                     GMLS_LinearAlgebra::batchLUFactorize(_pm, _RHS.data(), RHS_dim_0, RHS_dim_1, _P.data(), P_dim_1, P_dim_0, this_num_cols + added_coeff_size, this_num_cols + added_coeff_size, max_num_rows + _added_alpha_size, this_batch_size);
-                     Kokkos::Profiling::popRegion();
-                }
-            } else {
-                // Solve normally for no-constraint cases
-                if (_nontrivial_nullspace || _dense_solver_type == DenseSolverType::SVD) {
-                    Kokkos::Profiling::pushRegion("SVD Factorization");
-                    GMLS_LinearAlgebra::batchSVDFactorize(_pm, _P.data(), P_dim_0, P_dim_1, _RHS.data(), RHS_dim_0, RHS_dim_1, max_num_rows, this_num_cols, max_num_rows, this_batch_size);
-                    Kokkos::Profiling::popRegion();
-                } else if (_dense_solver_type == DenseSolverType::LU) {
+            if (_dense_solver_type == DenseSolverType::LU) {
                     Kokkos::Profiling::pushRegion("LU Factorization");
-                    GMLS_LinearAlgebra::batchLUFactorize(_pm, _RHS.data(), RHS_dim_0, RHS_dim_1, _P.data(), P_dim_1, P_dim_0, this_num_cols + added_coeff_size, this_num_cols + added_coeff_size, max_num_rows + _added_alpha_size, this_batch_size);
+                    GMLS_LinearAlgebra::batchQRPivotingSolve<layout_right,layout_left,layout_right>(_pm, _RHS.data(), RHS_dim_0, RHS_dim_1, _P.data(), P_dim_1, P_dim_0, this_num_cols + added_coeff_size, this_num_cols + added_coeff_size, max_num_rows + _added_alpha_size, this_batch_size);
                     Kokkos::Profiling::popRegion();
+            } else {
+                Kokkos::Profiling::pushRegion("QR+Pivoting Factorization");
+                if (_constraint_type != ConstraintType::NO_CONSTRAINT) {
+                     GMLS_LinearAlgebra::batchQRPivotingSolve<layout_right,layout_right,layout_right>(_pm, _RHS.data(), RHS_dim_0, RHS_dim_1, _P.data(), P_dim_1, P_dim_0, this_num_cols + added_coeff_size, this_num_cols + added_coeff_size, max_num_rows + _added_alpha_size, this_batch_size);
                 } else {
-                    Kokkos::Profiling::pushRegion("QR Factorization");
-                    GMLS_LinearAlgebra::batchQRFactorize(_pm, _P.data(), P_dim_0, P_dim_1, _RHS.data(), RHS_dim_0, RHS_dim_1, max_num_rows, this_num_cols, max_num_rows, this_batch_size);
-                    Kokkos::Profiling::popRegion();
+                    GMLS_LinearAlgebra::batchQRPivotingSolve<layout_right,layout_right,layout_right>(_pm, _P.data(), P_dim_0, P_dim_1, _RHS.data(), RHS_dim_0, RHS_dim_1, max_num_rows, this_num_cols, max_num_rows, this_batch_size);
                 }
+                Kokkos::Profiling::popRegion();
             }
 
             _pm.CallFunctorWithTeamThreadsAndVectors<ComputePrestencilWeights>(*this, this_batch_size);
@@ -506,14 +490,13 @@ void GMLS::operator()(const AssembleStandardPsqrtW&, const member_type& teamMemb
         // don't need to cast into scratch_matrix_left_type since the matrix is symmetric
         scratch_matrix_right_type M(_RHS.data()
             + TO_GLOBAL(local_index)*TO_GLOBAL(RHS_dim_0)*TO_GLOBAL(RHS_dim_1), RHS_dim_0, RHS_dim_1);
-        KokkosBatched::TeamVectorGemmInternal<KokkosBatched::Algo::Gemm::Unblocked>
+        KokkosBatched::TeamVectorGemm<member_type,KokkosBatched::Trans::Transpose,KokkosBatched::Trans::NoTranspose,KokkosBatched::Algo::Gemm::Unblocked>
 	      ::invoke(teamMember,
-	    	   this_num_cols, this_num_cols, this_num_rows, 
 	    	   1.0,
-	    	   PsqrtW.data(), PsqrtW.stride(1), PsqrtW.stride(0),
-	    	   PsqrtW.data(), PsqrtW.stride(0), PsqrtW.stride(1),
+	    	   PsqrtW,
+	    	   PsqrtW,
 	    	   0.0,
-	    	   M.data(), M.stride(0), M.stride(1));
+	    	   M);
         teamMember.team_barrier();
 
         // Multiply PsqrtW with sqrt(W) to get PW
@@ -638,14 +621,13 @@ void GMLS::operator()(const ComputeCoarseTangentPlane&, const member_type& teamM
     this->createWeightsAndPForCurvature(teamMember, delta, thread_workspace, PsqrtW, w, _dimensions, true /* only specific order */);
 
     // create PsqrtW^T*PsqrtW
-    KokkosBatched::TeamVectorGemmInternal<KokkosBatched::Algo::Gemm::Unblocked>
+    KokkosBatched::TeamVectorGemm<member_type,KokkosBatched::Trans::Transpose,KokkosBatched::Trans::NoTranspose,KokkosBatched::Algo::Gemm::Unblocked>
 	  ::invoke(teamMember,
-		   _dimensions, _dimensions, this->getNNeighbors(target_index), 
 		   1.0,
-		   PsqrtW.data(), PsqrtW.stride(1), PsqrtW.stride(0),
-		   PsqrtW.data(), PsqrtW.stride(0), PsqrtW.stride(1),
+		   PsqrtW,
+		   PsqrtW,
 		   0.0,
-		   PTP.data(), PTP.stride(0), PTP.stride(1));
+		   PTP);
     teamMember.team_barrier();
 
     // create coarse approximation of tangent plane in first two rows of T, with normal direction in third column
@@ -719,14 +701,13 @@ void GMLS::operator()(const AssembleCurvaturePsqrtW&, const member_type& teamMem
         scratch_matrix_right_type M(_RHS.data()
             + TO_GLOBAL(local_index)*TO_GLOBAL(RHS_dim_0)*TO_GLOBAL(RHS_dim_1), RHS_dim_0, RHS_dim_1);
         // Assemble matrix M
-        KokkosBatched::TeamVectorGemmInternal<KokkosBatched::Algo::Gemm::Unblocked>
+        KokkosBatched::TeamVectorGemm<member_type,KokkosBatched::Trans::Transpose,KokkosBatched::Trans::NoTranspose,KokkosBatched::Algo::Gemm::Unblocked>
           ::invoke(teamMember,
-               manifold_NP, manifold_NP, this_num_neighbors,
                1.0,
-               CurvaturePsqrtW.data(), CurvaturePsqrtW.stride(1), CurvaturePsqrtW.stride(0),
-               CurvaturePsqrtW.data(), CurvaturePsqrtW.stride(0), CurvaturePsqrtW.stride(1),
+               CurvaturePsqrtW,
+               CurvaturePsqrtW,
                0.0,
-               M.data(), M.stride(0), M.stride(1));
+               M);
         teamMember.team_barrier();
 
         // Multiply PsqrtW with sqrt(W) to get PW
@@ -1142,14 +1123,13 @@ void GMLS::operator()(const AssembleManifoldPsqrtW&, const member_type& teamMemb
         scratch_matrix_right_type M(_RHS.data()
             + TO_GLOBAL(local_index)*TO_GLOBAL(RHS_dim_0)*TO_GLOBAL(RHS_dim_1), RHS_dim_0, RHS_dim_1);
         // Assemble matrix M
-        KokkosBatched::TeamVectorGemmInternal<KokkosBatched::Algo::Gemm::Unblocked>
+        KokkosBatched::TeamVectorGemm<member_type,KokkosBatched::Trans::Transpose,KokkosBatched::Trans::NoTranspose,KokkosBatched::Algo::Gemm::Unblocked>
 	      ::invoke(teamMember,
-	    	   this_num_cols, this_num_cols, this_num_rows, 
 	    	   1.0,
-	    	   PsqrtW.data(), PsqrtW.stride(1), PsqrtW.stride(0),
-	    	   PsqrtW.data(), PsqrtW.stride(0), PsqrtW.stride(1),
+	    	   PsqrtW,
+	    	   PsqrtW,
 	    	   0.0,
-	    	   M.data(), M.stride(0), M.stride(1));
+	    	   M);
         teamMember.team_barrier();
 
 
