@@ -234,7 +234,9 @@ public:
     int getNeighborHost(int target_index, int neighbor_num) const {
         compadre_assert_debug((!_needs_sync_to_host) 
                 && "Stale information in host_cr_neighbor_lists. Call CopyDeviceDataToHost() to refresh.");
-        return _host_cr_neighbor_lists(_row_offsets(target_index)+neighbor_num);
+        compadre_assert_debug((neighbor_num<_host_number_of_neighbors_list(target_index))
+                && "neighor_num exceeds number of neighbors for this target_index.");
+        return _host_cr_neighbor_lists(_host_row_offsets(target_index)+neighbor_num);
     }
 
     //! Offers N(i,j) indexing where N(i,j) is the index of the jth neighbor of i (device)
@@ -317,7 +319,7 @@ NeighborLists<view_type_1d> Convert2DToCompressedRowNeighborLists(view_type_2d n
 
     
     // this will calculate row offsets
-    auto nla(CreateNeighborLists(new_cr_neighbor_lists, new_number_of_neighbors_list));
+    auto nla = CreateNeighborLists(new_cr_neighbor_lists, new_number_of_neighbors_list);
     auto cr_data = nla.getNeighborLists();
 
     // if device_execution_space can access this view, then write directly into the view
